;---------------------------------------------------
; TI-Menu.inc
; Re-usable menu code for CrASH.
; (c) 1999 Vasantha Crabb <_merlin@operamail.com>
;
; See file TI-Menu.txt for more information.
;
;  last modified: 03/17/2002 16:03 by Burkart
;
;---------------------------------------------------

; These macros address the menu data structure
#DEFINE MENU_LEN        ix+$00
#DEFINE MENU_DEFAULT    ix+$01
#DEFINE MENU_SEL        ix+$02
#DEFINE MENU_TITLE      ix+$03

;---------------------
; SUB for handling a menu
;---------------------

RunMenu:
    push    hl                  ; Get address menu structure it into ix
    pop     ix

    push    hl
    ROM_CALL(CLEARLCD)          ; Clear the text screen
    pop     hl

    ld      b,(hl)              ; Get the number of items in b

    inc     hl                  ; Point hl at the title
    inc     hl
    inc     hl

    ld      de,$0000            ; Move to the top left corner of LCD
    ld      (CURSOR_ROW),de
    set     3,(iy+$05)          ; White on black for title
    ROM_CALL(D_ZT_STR)          ; Display the title

    ld      c,1
    res     3,(iy+$05)          ; Black on white for the options
_MenuDispLoop:
    ld      a,c                 ; Put item number in a
    call    _MenuDispItemNo     ; Show item number

    ROM_CALL(D_ZT_STR)          ; Display the item's text
    inc     c                   ; Move down a line
    djnz    _MenuDispLoop       ; Loop

    ld      a,(MENU_DEFAULT)    ; Start on default item
    call    _MenuNewItem        ; Get the marker onto the screen

    ret

;-------------------------------

_MenuHandLoop:
    ; GET_KEY code in a
    push    hl                  ; Get address menu structure it into ix
    pop     ix

    cp      G_UP                ; Select previous menu item
    call    z,_MenuUpHandler

    cp      G_DOWN              ; Select next menu item
    call    z,_MenuDwnHandler

    cp      G_ENTER             ; User's made up their mind
    jr      z,_MenuRetItem

    cp      G_ZOOM              ; User wants out
    jr      z,_MenuRetZero

    cp      G_WINDOW            ; User wants to get to main menu
    jr      z,_MenuRet254

    cp      G_LEFT              ; next page
    jr      z,_MenuRet252

    cp      G_RIGHT             ; next page
    jr      z,_MenuRet253

    ; now let's try to get keys "1".."9":

    sub     10                  ; A: "3"=8 "6"=9 "9"=10 "2"=16 "5"=17 "6"=18 "1"=24 "4"=25 "7"=26
    ld      b, 3
_Menu_get_1__9_loop:
    sub     8                   ; B=3, A: "3"=0 "6"=1 "9"=2    "2"=8 "5"=9 "6"=10 "1"=16 "4"=17 "7"=18
                                ; B=2, A: "2"=0 "5"=1 "6"=2    "1"=8 "4"=9 "7"=10
                                ; B=1, A: "1"=0 "4"=1 "7"=2
    cp      3
    jr      c, _Menu_get_1__9_check2 ; A < 3 -> get number
    djnz    _Menu_get_1__9_loop
    jr      _MenuRet255         ; other key -> exit

_Menu_get_1__9_check2:
    ; number(1..9) = 3*A + B = A+A+A+B
    ld      c, a
    add     a, a
    add     a, c
    add     a, b
    ; now A contains 1..9

    ld      b,a                 ; Save this for now
    ld      a,(MENU_LEN)        ; See if it's on the menu
    cp      b
    jr      c,_MenuRet255       ; Return if it isn't
    ld      a,b                 ; Otherwise we return
    ret

_MenuRetItem:
    ld      a,(MENU_SEL)        ; Put the selection in a
    ret                         ; Return

_MenuRetZero:
    xor     a
    ret                         ; Return

_MenuRet254:
    ld      a, 254
    ret

_MenuRet253:                    ; right
    ld      a, 253
    ret

_MenuRet252:                    ; left
    ld      a, 252
    ret

;---------------------
; SUB for handling DOWN key on a menu
;---------------------

_MenuDwnHandler:
    ld      a,(MENU_SEL)        ; Get the current menu item
    cp      (MENU_LEN)          ; Return if we're on the last item
    jr      z,_MenuRet255

    push    af                  ; Save item number
    call    _MenuDispItemNo     ; Show item number (normal text)

    pop     af                  ; Restore item number
    inc     a                   ; Increment menu item
_MenuNewItem:                   ; For code re-use
    ld      (MENU_SEL),a        ; Store the new menu item

    set     3,(iy+$05)          ; White text on black
    call    _MenuDispItemNo     ; Show item number (inverse text)
    res     3,(iy+$05)          ; Black text on white

_MenuRet255:
    ld      a, 255
    ret

;---------------------
; SUB for handling UP key on a menu
;---------------------

_MenuUpHandler:
    ld      a,(MENU_SEL)        ; Get the current menu item
    cp      1                   ; Return if we're on the first item
    jr      z,_MenuRet255

    push    af                  ; Save item number
    call    _MenuDispItemNo     ; Show item number (normal text)

    pop     af                  ; Restore item number
    dec     a                   ; Decrement menu item
    jr      _MenuNewItem        ; Re-use code

;---------------------
; SUB for showing menu item number
;---------------------

_MenuDispItemNo:
    ld      d,$00               ; Cursor column 1
    ld      e,a                 ; Cursor row a
    ld      (CURSOR_ROW),de
    add     a,'0'               ; Convert item number to character
    ROM_CALL(TX_CHARPUT)        ; Put the character on the screen
    ld      a,':'               ; Print a colon
    ROM_CALL(TX_CHARPUT)
    ret                         ; Return

