
;
;       input2.inc
;       TI-82 version
;
; Displays an "Eingabefeld"
;
; Last modified: 07/15/2001 mmddyy 15:39 by Burkart (general routines; Eingabefeld demo)
;                03/10/2002 mmddyy 15:14 by Burkart (separating key_check and DisplayString)
;                03/19/2002 mmddyy 11:03 by Burkart
;
; call input2_DisplayString to display/initialize the Eingabefeld
; call input2_key_check with GET_KEY code in A to update the Eingabefeld
;

input2_CURSOR_CHAR = $5F
input2_BLANK_CHAR = $E1

input2_key_check:
        ; GET_KEY code has to be in a
        cp      G_ENTER                         ; "ENTER" -> exit
        jp      z, input2_exit
        cp      G_WINDOW                        ; "WINDOW" -> exit
        jp      z, input2_exit
        cp      G_ZOOM                          ; "ZOOM" -> exit
        jp      z, input2_exit
        cp      G_DEL                           ; "DEL" -> StrLen--
        jp      z, input2_Backspace
        cp      G_CLEAR                         ; "CLEAR" -> clear string
        jp      z, input2_ClearString
        cp      G_ALPHA                         ; "ALPHA" -> change key mode
        jr      z, input2_alpha
        cp      G_2ND                           ; "2nd" -> change key mode
        jr      z, input2_2nd

        cp      $0A                             ; is $0A < scancode < $30?
        jp      c, input2_exit2                      ; if not exit
        cp      $30
        jp      nc, input2_exit2

        sub     $0A                             ; let hl point to the character of the pressed key
        ld      b, $00
        ld      c, a
        ld      a, (input2_Var_KeyMode)
        cp      1
        jr      z, input2_key_map1
        cp      2
        jr      z, input2_key_map2
        cp      3
        jr      z, input2_key_map3
        jp      input2_exit2                     ; illegal key mode -> return to caller
input2_key_map1:
        ld      hl, input2_Map_KeyToChar1
        jr      input2_key_anymap
input2_key_map2:
        ld      hl, input2_Map_KeyToChar2
        jr      input2_key_anymap
input2_key_map3:
        ld      hl, input2_Map_KeyToChar3
       ;jr      input2_key_anymap
input2_key_anymap:
        add     hl, bc

        ld      a, (hl)                         ; was it a special key (i.e. ON, ENTER)?
        or      a
        jp      z, input2_exit2                 ; if so, RETurn to caller
        ; TI-ASCII ;-) code is now in a

        ld      d, a                            ; save TI-ASCII code in d
        ld      a, (input2_Var_StrLen)          ; StrLen already == MaxLen?
        ld      hl, input2_TiML_MaxLen
        cp      (hl)
        jp      z, input2_exit2                 ; if so, RETurn to caller
        inc     a                               ; StrLen++
        ld      (input2_Var_StrLen), a
        ld      c, a                            ; move pointer to next byte
        ld      b, 0
        ld      hl, input2_Var_String-1
        add     hl, bc
        ld      (hl), d                         ; save new byte
        jr      input2_DisplayString            ; display Eingabefeld


input2_alpha:
        ; TODO: first check if key mode is allowed
        ld      a, (input2_Var_KeyMode)         ; get old key mode
        cp      1                               ; was it "capital letter" mode?
        jr      z, input2_alpha_setlower        ; then set mode to "lower case letter"
        ld      a, 1                            ; set mode to "capital letter"
        ld      (input2_Var_KeyMode), a
        jp      input2_exit2                    ; RETurn to caller
input2_alpha_setlower:
        ld      a, 2                            ; set mode to "lower case letter"
        ld      (input2_Var_KeyMode), a
        jp      input2_exit2                    ; RETurn to caller

input2_2nd:
        ; TODO: first check if key mode is allowed
        ld      a, 3                            ; set mode to "numbers"
        ld      (input2_Var_KeyMode), a
        jp      input2_exit2                    ; RETurn to caller

input2_Backspace:
        ld      a, (input2_Var_StrLen)
        or      a
        jp      z, input2_exit2                 ; already StrLen=0 -> RETurn to caller
        dec     a                               ; StrLen--
        ld      (input2_Var_StrLen), a
        jr      input2_DisplayString            ; diplay Eingabefeld

input2_ClearString:
        xor     a                               ; StrLen := 0
        ld      (input2_Var_StrLen), a
;       jr      input2_DisplayString            ; display Eingabefeld

input2_DisplayString:
        ld      hl, input2_TiML_StartPos        ; set cursor position
        call    LD_HL_MHL
        ld      (CURSOR_POS), hl

        ld      a, (input2_Var_StrLen)
        ld      hl, input2_TiML_DispLen
        cp      (hl)
        jr      z, input2_DispStr_eqDispLen
        jr      c, input2_DispStr_smDispLen
        ; StrLen > DispLen
        ld      hl, input2_TiML_MaxLen
        cp      (hl)
        jr      nz, input2_DispStr_biDLneMaxLen
        ; StrLen > DispLen & StrLen = MaxLen (case IV)
        ld      hl, input2_Var_String+1         ; goto char offset StrLen-DispLen+1
        ld      b, 0
        ld      c, a
        add     hl, bc
        ld      a, (input2_TiML_DispLen)
        ld      c, a
        ld      d, a
        ld      a, l
        sub     c
        jr      nc, input2_Over1
        dec     h
input2_Over1:
        ld      l, a
        ld      a, d
        ld      de, input2_Buf_DispStr          ; copy to buffer
        dec     a                               ; copy DispLen-1 chars
        ld      c, a
        ldir                                    ; do it now!
        ld      hl, input2_TiML_DispLen         ; finally display it
        ROM_CALL(D_LT_STR)
        jr      input2_exit2                    ; RETurn to caller
input2_DispStr_eqDispLen:
        ; StrLen = DispLen
        ld      hl, input2_TiML_MaxLen
        cp      (hl)
        jr      z, input2_DispStr_eqDLeqMaxLen
        ; StrLen = DispLen & StrLen < MaxLen (case III)
input2_DispStr_biDLneMaxLen:
        ; StrLen > DispLen & StrLen < MaxLen (case III)
        ; a = (Var_StrLen)
        ld      hl, input2_Var_String+2         ; goto char offset StrLen-DispLen+2
        ld      b, 0
        ld      c, a
        add     hl, bc
        ld      a, (input2_TiML_DispLen)
        ld      c, a
        ld      d, a
        ld      a, l
        sub     c
        jr      nc, input2_Over2
        dec     h
input2_Over2:
        ld      l, a
        ld      a, d
        ld      de, input2_Buf_DispStr          ; copy to buffer
        sub     2                               ; copy DispLen-2 chars
        ld      c, a
        ldir                                    ; do it now!
        ld      a, input2_CURSOR_CHAR           ; add cursor
        ld      (de), a
        ld      hl, input2_TiML_DispLen         ; finally display it
        ROM_CALL(D_LT_STR)
        jr      input2_exit2                    ; RETurn to caller
input2_DispStr_eqDLeqMaxLen:
        ; StrLen = DispLen & StrLen = MaxLen (case II)
        ld      hl, input2_Var_StrLen
        ROM_CALL(D_LT_STR)
        jr      input2_exit2                    ; RETurn to caller
input2_DispStr_smDispLen:
        ; StrLen < DispLen (case I)
        ld      hl, input2_Var_StrLen           ; display string
        push    af
        ROM_CALL(D_LT_STR)
        pop     af
        ld      b, a                            ; b = StrLen
        ld      a, input2_CURSOR_CHAR           ; display cursor
        ROM_CALL(TX_CHARPUT)
        ld      a, (input2_TiML_DispLen)        ; a = DispLen
        sub     b                               ; a = DispLen - StrLen
        dec     a                               ; a = DispLen - StrLen - 1
        or      a                               ; a = 0 => 0 blanks? -> RETurn to caller
        jr      z, input2_exit2
        ld      b, a                            ; display blanks
        ld      a, input2_BLANK_CHAR
input2_DisplayBlank:
        ROM_CALL(TX_CHARPUT) ; warning: TX_CHARPUT might convert some control chars
        djnz    input2_DisplayBlank
;       jr      input2_exit2                    ; RETurn to caller

input2_exit2:
        xor     a
input2_exit:
        ret                                     ; return to caller



input2_Var_StrLen: ; Byte
        .db 0
input2_Var_String: ; 255 Bytes
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

input2_TiML_StartPos:
        .dw $0000
input2_TiML_AllowedChars:
        .db 0
        ; allowed = bit cleared
        ; bit     meaning
        ;  0   numbers
        ;  1   lower case letters
        ;  2   capital letters
        ; 4-7  special characters (all bits have to be set)
input2_TiML_MaxLen:
        .db 8
input2_TiML_DispLen:
        .db 8
        .db $CF ; arrow to the left
input2_Buf_DispStr:
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 ; 15

input2_Var_KeyMode:
        .db 1

input2_Map_KeyToChar1:
        .db '"',"WRMH",0,0,"?,VQLG",0,0,":ZUPKFC",0," YTOJEB",0,0,"XSNIDA"
input2_Map_KeyToChar2:
        .db '"',"wrmh",0,0,"?@vqlg",0,0,".zupkfc",0," ytojeb",0,0,"xsnida"
input2_Map_KeyToChar3:
        .db 0,0,0,0,0,0,0,"-369",0,0,0,0,".258",0,0,0,0,"0147,",0,0,0,0,0,0,0,0,0,0

