
;
;   options.inc
;
;   This file contains routines dealing with the options menu
;
;   last modified: 07/18/2002 mmddyy 23:04 by Burkart
;
; List of routines:
;
; - call options_menu
; - call options_wait_key_and_handle_uc
;


;-----------------------------------------------
; call options_menu
;-----------------------------------------------
; modified: ???
;-----------------------------------------------
; stack: ??? byte + caller address
;-----------------------------------------------

options_menu:
        res     2, (IY+$0D)                     ; disable auto scrolling
        ld      hl, menu_opt_main               ; initialize menu
        call    RunMenu
        set     2, (IY+$0D)                     ; enable auto scrolling

options_menu_loop1:
        call    options_wait_key_and_handle_uc  ; wait for keypress
        
        ld      hl, menu_opt_main               ; update menu
        call    _MenuHandLoop

        or      a                               ; return=0 (ZOOM) -> exit
        jr      z, options_menu_exit
        cp      1                               ; return=1 ("Logout") -> logout
        jr      z, options_menu_logout
        cp      2                               ; return=2 ("Ping C") -> ping C=
        jr      z, options_menu_ping
        cp      3                               ; return=3 ("Exit / back") -> exit
        jr      z, options_menu_exit

        jr      options_menu_loop1              ; something else -> loop again

options_menu_logout:
        pop     af                              ; clean up stack (caller address)
        jp      main_logout                     ; logout

options_menu_ping:
        ROM_CALL(CLEARLCD)                      ; clear LCD
        ld      hl, $0603                       ; move cursor
        ld      (CURSOR_POS), hl
        ld      hl, str_ping                    ; display "Ping"
        ROM_CALL(D_ZT_STR)

        ; send <TI ping>:

        call    send_start                      ; send START
        ld      hl, tiucpacket_ping             ; template packet
        ld      b, 3                            ; consists of 3 bytes
        call    send_packet_to_uc               ; send to C

        ; C should reply with <C ACK>:

        call    wait_for_start_with_timeout     ; wait for START
        jp      c, error_start_timedout         ; DEVELOPMENT

        call    receive_packet                  ; receive packet
        ld      hl, tiucpacket_ack              ; check for <C ACK>
        ld      b, 3                            ; the first 3 byte have to be identical
        call    check_received_packet           ; perform the comparison
        jp      c, error_packets_differ         ; DEVELOPMENT

        ld      hl, $0604                       ; move cursor
        ld      (CURSOR_POS), hl
        ld      hl, str_pong                    ; display "Pong"
        ROM_CALL(D_ZT_STR)

        call    options_wait_key_and_handle_uc  ; wait for keypress
        jr      options_menu                    ; return to main options menu

options_menu_exit:
        xor     a                               ; (flag_new_radio_packet) := true
        ld      (var_flag_new_radio_packet), a  ; ...in order to refresh the content display

        ret                                     ; return to main loop

;-----------------------------------------------
; call options_wait_key_and_handle_uc
;-----------------------------------------------
; modified: ???
;-----------------------------------------------
; stack: ??? byte + caller address
;-----------------------------------------------
; output: a - GET_KEY code of pressed key
;-----------------------------------------------

options_wait_key_and_handle_uc:
        in      a, (P_LINK)                     ; check for START
        sra     a
        call    nc, handle_main_start           ; START received -> handle it

        halt                                    ; save power
        call    GET_KEY                         ; wait for keypress
        or      a                               ; no key pressed -> loop again
        jr      z, options_wait_key_and_handle_uc

        ret                                     ; return with keycode in a

;-----------------------------------------------

menu_opt_main:
        .db 3                           ; 3 items
        .db 1                           ; default item 1
        .db 0                           ; dummy byte
        .db " Client Options ",0        ; title
        .db "Logout",0                  ; item 1
        .db "Ping ",$C3,"C",0           ; item 2
        .db "Exit / back",0             ; item 3

