
;
;   supply.inc
;
;   This file contains supply routines
;
;   last modified: 07/18/2002 mmddyy 21:36 by Burkart
;
; List of routines:
;
; - call wait_1sec
; - call disp_hex
; - call wait_for_keypress
; - call disp_hex_table
; - call debug_wait
;


;-----------------------------------------------
; call wait_1sec
;-----------------------------------------------
; wait ABOUT 1 second
;-----------------------------------------------
; modified: N/A
;-----------------------------------------------
; stack: 4 byte + caller address
;-----------------------------------------------

wait_1sec:
        push    bc
        ld      b, $00
wait_1sec_loop1:
        push    bc
        ld      b, $00
wait_1sec_loop2:
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        djnz    wait_1sec_loop2
        pop     bc
        djnz    wait_1sec_loop1
        pop     bc
        ret

;-----------------------------------------------
; call disp_hex
;-----------------------------------------------
; Displays the byte at (hl) as two char hex
;-----------------------------------------------
; input: (hl) - byte to display
;-----------------------------------------------
; modified: N/A?
;-----------------------------------------------
; stack: 6 byte (+ call address)
;-----------------------------------------------

disp_hex:
        push    hl; NOT NECESSARY?
        push    bc
        push    af
        ld      b, 2
        xor     a
disp_hex_hex_loop:
        rld
        push    af
        add     a, '0'
        cp      '9'+1
        jr      c, disp_hex_number
        add     a, 'A'-'0'-10
disp_hex_number:
        ROM_CALL(TX_CHARPUT)
        pop     af
        djnz    disp_hex_hex_loop
        rld
        pop     af
        pop     bc
        pop     hl ; NOT NECESSARY?
        ret

;-----------------------------------------------
; call wait_for_keypress
;-----------------------------------------------
; loops until keypress occurs
; this routine should not be called while there
; might be TI<->uC activity, as while waiting
; for a keypress no TI<->uC communication is
; handled
;-----------------------------------------------
; modified: N/A
;-----------------------------------------------
; stack: 2 byte + caller address
;-----------------------------------------------

wait_for_keypress:
        push    af                              ; save used registers

wait_for_keypress_loop:
        halt                                    ; wait for keypress while saving power
        call    GET_KEY                         ; get keypad data
        or      a                               ; any key pressed?
        jr      z, wait_for_keypress_loop       ; no, loop again

        pop     af                              ; clear up stack (saved registers)
        ret                                     ; return to caller

;-----------------------------------------------
; call disp_hex_table
;-----------------------------------------------
; ...
;-----------------------------------------------
; modified: N/A (?)
;-----------------------------------------------
; stack: ... + caller address
;        + x byte (disp_hex)
;-----------------------------------------------
;
;disp_hex_table:
;       push    hl                              ; save used registers
;       push    bc
;       push    af
;       res     2, (IY+$0D)                     ; disable auto scrolling
;
;       ROM_CALL(CLEARLCD)                      ; clear LCD
;       ld      hl, $0000                       ; reset cursor
;       ld      (CURSOR_POS), hl
;
;       ld      b, 8                            ; display 8 lines
;disp_hex_table_lines_loop:
;       push    bc
;
;       ld      a, ' '
;       ld      b, 4
;disp_hex_table_line_loop1:
;       ; hl points to byte-to-show
;       call    disp_hex                        ; display byte as hex
;       inc     hl                              ; advance pointer
;       push    hl ; NOT NECESSARY?
;       ROM_CALL(TX_CHARPUT)                    ; display blank
;       pop     hl ; NOT NECESSARY?
;       djnz    disp_hex_table_line_loop1       ; display all 4 bytes
;
;       dec     hl                              ; let hl point to first byte of this line again
;       dec     hl
;       dec     hl
;       dec     hl
;
;       ld      b, 4
;disp_hex_table_line_loop2:
;       ; hl points to byte-to-show
;
;       ld      a, (hl)                         ; display byte as single character
;       push    hl ; NOT NECESSARY?
;       ROM_CALL(TX_CHARPUT)
;       pop     hl ; NOT NECESSARY?
;       inc     hl                              ; advance pointer
;       djnz    disp_hex_table_line_loop2       ; display all 4 bytes
;
;       pop     bc
;       djnz    disp_hex_table_lines_loop       ; display next line
;
;       set     2, (IY+$0D)                     ; enable auto scrolling
;       pop     af                              ; clean up stack (saved registers)
;       pop     bc
;       pop     hl
;       ret                                     ; return to caller

#ifdef _USE_DELAYS
;-----------------------------------------------
; call debug_wait
;-----------------------------------------------
; waits a little (about 7ms?)
;-----------------------------------------------
; modified: N/A
;-----------------------------------------------
; stack: 2 byte + caller address
;-----------------------------------------------

debug_wait:
        push    bc
        ld      b, 0
debug_wait_loop:
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        djnz    debug_wait_loop
        pop     bc
        ret
#endif

